/*********************************************************************
MODULE: 	pbitdraw.c
PROGRAMMERS:    John White and Yaoen Zhang
DESCRIPTION:    This module is used to create a customized 8x8 texture
		map for tiling a rectangle.  Functions include:
		create_tiling() and init_info().  
*******************************************************************/

#include "p2.h"           /* Header contains standard X includes 
			   and project-specific data structures */ 
				    
/*********************************************************************
FUNCTION:       create_tiling
PURPOSE:        Allow a user to create a customized texture map.
		This function is called as a popup.  Its widget 
		structure is as follows:  
					toplevel
					    ^ 
					   form 
			==========================================	
                        =   command_box   =      tiling_box      =
			=   -----------   =                      =
			=   next_command  =                      =
			=   prev_command  =                      =
                        =   write_command =                      =
		        =   clear_command =                      =
			=   rev_command   =				
			=   quit_command  =                      =
                        =   ------------  =                      =
			=   viewing_box   =                      =
                        =                 =			 =
			==========================================
			     
		The command_box contains the command buttons.
		The tiling_box contains a grid where users click
		    on rectangles which correspond to individual
		    pixels.
		The viewing_box contains a pixel by pixel rendering
		    of the texture being created in the tiling_box.

PARAMETERS:	
toplevel widget: The controlling popup widget shell which is passed
		 from the main program, projmain.c
CALLED BY:       PopupPrompt in projmain.c, which is a callback to
		 the edit command button.
SIDE EFFECTS:    A write_command will record the texture in the
		 global variable tile, which is of Tile_type.
BUGS:            Resizing of this widget will destroy the grid.
*****************************************************************/
extern int current_map;

void create_tiling(toplevel,data)
Widget toplevel;
canvas_data *data;
{

extern 
   render_grid(),    /* displays rectangular grid                    */
   copy_tiling(),    /* copies a source Tile_type to a destination
				Tile_type			     */ 
   start_select_square(),  /* fills a rectangle and sets corresponding
				pixel in the viewing box             */
   move_select_square(),  /* fills a rectangle and sets corresponding
				pixel in the viewing box             */
   stop_select_square(),  /* fills a rectangle and sets corresponding
				pixel in the viewing box             */
   save_image(),     /* saves current texture mapping, an event 
				handler for write_command	     */		 
   next_image(),     /* selects next texture map for modification    */

   prev_image(),     /* selects previous texture map for modification    */

   clear_tile(),     /* clears current texture map,an event handler 
				for clear_command		     */ 
   rev_tile(),       /* makes an inverse image of the current tiling */
   put_message(),    /* put message on help window  	             */
   destroypopup();   /* destroys this popup application              */ 

void  
      init_info();   /* sets initial values for info structure        */
void  draw_maps();   /* draw the tile pattern on main window */

Widget 
   form,       	     /* form widget containing 3 box widgets         */
   command_box,      /* box widget containing clear, quit, and write */
   viewing_box,      /* box widget which displays bitmap             */
   tiling_box,       /* box widget where bitmap is created           */
   next_command,     /* command to select next texture map           */
   prev_command,     /* command to select previous texture map       */
   rev_command,	     /* command to draw inverse image 		     */
   write_command,    /* command widget to save current design        */
   clear_command,    /* command widget clear design                  */
   quit_command;     /* command widget that exits the application    */

Arg  args[2];        /* array of arguments to change resources       */
char mstr[MAXSTRING];

      
XGCValues gcv;       /* structure to hold graphic context parameters */

GC 
    gc_tb,           /* graphic context for tiling box                */
    gc_vb;           /* graphic context for viewing box		      */   
Info *parms;         /* pointer to structure containing application
				specific information		     */



    parms = (Info *) malloc (sizeof(Info));

    /* create and initialize form widget */
    form = XtCreateManagedWidget("form", formWidgetClass,
			 		toplevel, NULL, ZERO); 
    XtAddEventHandler(form, ExposureMask, FALSE,render_grid, parms);
    /*
    XtAddCallback(form,XtNdestroyCallback,draw_maps,data);
    */

    /* create command box */
    command_box = XtCreateManagedWidget("command_box", boxWidgetClass,
					 form, NULL, ZERO); 

    /* create and initialize next_command button */
    next_command = XtCreateManagedWidget("  Next  ",
		 commandWidgetClass, command_box, NULL, ZERO);
    XtAddEventHandler(next_command, ButtonPressMask, FALSE, 
					 next_image, parms);

    /* create and initialize prev_command button */
    prev_command = XtCreateManagedWidget("Previous",
		 commandWidgetClass, command_box, NULL, ZERO);
    XtAddEventHandler(prev_command, ButtonPressMask, FALSE, 
					 prev_image, parms);

    /* create and initialize write_command button */
    write_command = XtCreateManagedWidget("  Write ", commandWidgetClass,
	 				 command_box, NULL, ZERO);
    XtAddEventHandler(write_command, ButtonPressMask, FALSE, 
					 save_image, parms);

    /* create and initialzie clear_command button */
    clear_command = XtCreateManagedWidget("  Clear ", commandWidgetClass,
	 				command_box, NULL, ZERO);
    XtAddEventHandler(clear_command, ButtonPressMask, FALSE, 
					   clear_tile, parms);

    /* create and initialzie rev_command button */
    rev_command = XtCreateManagedWidget("Inverse ", commandWidgetClass,
	 				command_box, NULL, ZERO);
    XtAddEventHandler(rev_command, ButtonPressMask, FALSE, 
					   rev_tile, parms);

    /* create and initialize quit_command button */
    quit_command = XtCreateManagedWidget("  Quit  ", commandWidgetClass,
					 command_box, NULL, ZERO);
    XtAddCallback(quit_command, XtNcallback, destroypopup, data);


    /* create viewing box and position in beneath the command box */
    XtSetArg(args[0], XtNfromVert, command_box);
    viewing_box = XtCreateManagedWidget("viewing_box", boxWidgetClass,
		form, args, ONE);
    
    /* create tiling box and position to the right of the command box*/
    XtSetArg(args[0], XtNfromHoriz, command_box);
    tiling_box = XtCreateManagedWidget("tiling_box", boxWidgetClass,
		form, args, ONE);
    XtAddEventHandler(tiling_box, ButtonPressMask, FALSE, start_select_square,
			parms);
    XtAddEventHandler(tiling_box, ButtonMotionMask, FALSE, move_select_square,
			parms);
    XtAddEventHandler(tiling_box, ButtonReleaseMask, FALSE, stop_select_square,
			parms);

    /* adjust size of form widget */
    XtSetArg(args[0], XtNwidth, 270);
    XtSetArg(args[1], XtNheight, 260);
    XtSetValues(form, args, 2);

    /* adjust size of tiling box */
    XtSetArg(args[0], XtNwidth, 245);
    XtSetArg(args[1], XtNheight, 260);
    XtSetValues(tiling_box, args, 2);

    /* adjust size of command box */
    XtSetArg(args[0], XtNwidth, 50);
    XtSetArg(args[1], XtNheight, 200);
    XtSetValues(command_box, args, 2);

    /* adjust size of viewing box */
    XtSetArg(args[0], XtNwidth, 55); 
    XtSetArg(args[1], XtNheight,  60);
    XtSetValues(viewing_box, args, 2);

    /* position form widget */ 
    XtSetArg(args[0], XtNx, 0);
    XtSetArg(args[1], XtNy, 0);
    XtSetValues(form, args, 2);

    XtRealizeWidget(toplevel);

    /* Set graphics context to be XOR drawable */
    gcv.function = GXcopy;
    gcv.line_style = LineSolid;
    gcv.foreground = 1;
    gcv.background = 0;

    /* tiling box graphics context */
    gc_tb = XCreateGC(XtDisplay(tiling_box), XtWindow(tiling_box),
		 GCFunction | GCForeground | GCBackground 
		 | GCLineStyle,  &gcv);

    /* viewing box graphics context */
    gc_vb = XCreateGC(XtDisplay(viewing_box), XtWindow(viewing_box),
		 GCFunction | GCBackground | GCForeground, &gcv);

    /* initialize parms with values of the various widgets */
    init_info(form,data->message,data->tilewin, tiling_box,
			viewing_box,gc_tb,gc_vb,parms);
    /* put a message saying tiling current tile */
    if(current_map == 4)
         sprintf(mstr,"Tiling the No. %d texture map", current_map + 2);
    else
         sprintf(mstr,"Tiling the No. %d texture map", 
			(current_map + 2)%MAX_TEXTURE_MAPS);
    put_message(data,mstr);
}

/*******************************************************************
FUNCTION:	init_info
PURPOSE:  	initialize pointer to Info structure with information
		concerning widgets to be used in subsequent events
PARAMETERS:     target   - main application widget (form)
	        w1, w2   - tiling_box and viewing_box widgets
		gc1, gc2 - graphics contexts for tiling_box and
			   viewing box, respectively	
		data     - pointer to Info structure which will hold
			   information obtained through this function
CALLED BY:	create_tiling()
SIDE EFFECTS:	none
**********************************************************************/
void init_info(target,mhelp,tilepat,w1,w2,gc1,gc2,data)
Widget target, mhelp, tilepat;
Widget w1, w2;
GC gc1, gc2;
Info *data;
{
  data->target = target;
  data->help = mhelp;
  data->tileview = tilepat; 
  data->w1 = w1;
  data->w2 = w2;
  data->gc1 = gc1;
  data->gc2 = gc2;
  data->w1_display = XtDisplay(w1); /* records tiling_box display i.d.*/
  data->w2_display = XtDisplay(w2); /* records viewing_box display i.d*/
  data->w1_window  = XtWindow(w1);  /* records tiling_box window i.d. */
  data->w2_window  = XtWindow(w2);  /* records viewing_box window i.d */
  data->write = NOT_SET;
  data->color_flag = NOWRITE;
  copy_tiling(tile[current_map],data->tmp_tile);
 /* makes local copy of global variable, tile */ 
}

/*
 *FUNCTION:     help_message
 *PARAMETERS:   toplevel -- the popup widget
 *              data -- canvas data structure
 *CALLED:       by PopupHelp()
 */
void help_message(toplevel,data)
Widget toplevel;
canvas_data *data;
{

   void quithelp();   /* destroys this popup application       */

   Widget
   hform,            /* form widget containing 3 box widgets         */
   helptext,         /* text widget to contain help message          */
   quit_command;     /* command widget that exits the application    */

   Arg  args[3];     /* array of arguments to change resources       */
    /* create and initialize form widget */
    hform = XtCreateManagedWidget("hform", formWidgetClass,
                                        toplevel, NULL, ZERO);
    /* adjust size of text widget */
    helptext=XtCreateManagedWidget("helptext",labelWidgetClass,
                hform, NULL,ZERO);
    /* create  the cancel button */
    quit_command=XtCreateManagedWidget("quit_command",commandWidgetClass,
                hform,NULL,ZERO);
    XtAddCallback(quit_command,XtNcallback,quithelp,NULL);
}
/*
 *FUNCTION:     quithelp
 *PURPOSE:      destroy the popup help widget
 *PARAMETERS:   toplevel -- the popup widget
 *              data -- canvas data structure
 *CALLED:       by PopupHelp()
 */
void quithelp(w,client,call)
  Widget w;
  XtPointer client, call;
{
        Widget form=XtParent(XtParent(w));
        XtDestroyWidget(form);
}
/*
 *FUNCTION:     draw_maps 
 *PURPOSE:      draw the tile pattern on the main window  
 *PARAMETERS:   w -- the form widget
 *              data -- canvas data structure
 *CALLED:       by PopupPrompt() 
 */
void draw_maps(w,data,call)
  Widget w;
  canvas_data *data;
  XtPointer call;
{
int i,j,k,l,m,temp;

XClearWindow(XtDisplay(data->tilewin),XtWindow(data->tilewin));
XSetFunction(XtDisplay(data->tilewin),data->gc,GXcopy);
 
for(i=0;i<FACES;i++)
 {
  temp=(i+1)%6;
  XDrawRectangle(XtDisplay(data->tilewin),XtWindow(data->tilewin),data->gc, 
  ((i%3)*40+8), ((i/3)*40+8), 32, 32);
  for(j=0;j<TILE_DIM;j++)
    for(k=0;k<TILE_DIM;k++)
       if(tile[i][j][k] == SET)
         for(l=0;l<UNIT_AREA-1;l++)
	    for(m=0;m<UNIT_AREA-1;m++)
		XDrawPoint(XtDisplay(data->tilewin),XtWindow(data->tilewin),
		  data->gc, ((temp%3)*40+8) + (j*(UNIT_AREA-1)) + l,
			    ((temp/3)*40+8) + (k*(UNIT_AREA-1)) + m);
		
  }
}
